/* Copyright (C) 2000 Andrei Zmievski
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <andrei@ispi.net>.
 */

#ifndef BALLOC_H
#define BALLOC_H 1

#ifdef LIBFLUX_BUILD
# include "types.h"
#else
# include <flux/types.h>
#endif

#define BALLOC_MEM_CHUNKS     256
#define BALLOC_BITS_IN_ULONG  (SIZEOF_UNSIGNED_LONG * 8)
#define BALLOC_BITMAP_SIZE    (BALLOC_MEM_CHUNKS / BALLOC_BITS_IN_ULONG)

#if SIZEOF_UNSIGNED_LONG == 4
# define BALLOC_MULSHIFT_AMOUNT 5
#elif SIZEOF_UNSIGNED_LONG == 8
# define BALLOC_MULSHIFT_AMOUNT 6
#endif

#define BALLOC_CHUNK_STORAGE(chunk)  ((byte *) chunk + sizeof(BCHUNK))

/* BT depends on BMEM, so we can't just include it at the top.
 * Instead we make half-assed definitions here. */

#ifndef _BT_DEFINED
#define _BT_DEFINED 1
typedef struct _bt_node
{
  u32 is_red : 1;
  void *key;
  void *data;
  struct _bt_node *left, *right;
}
BTN;

typedef struct _bt
{
  struct _bmem_mem *bmem;
  struct _bt_node head, z;
}
BT;
#endif

typedef struct _bmem_chunk
{
  byte start_looking_from_bit;
  byte start_looking_from_index;
  u16 allocated;
  ulong bitmap[BALLOC_BITMAP_SIZE];
} BCHUNK;

typedef struct _bmem_mem
{
  BT *eligible_chunks;
  BT *saturated_chunks;
  BCHUNK *current_chunk;
  size_t block_size;
  size_t storage_size;
  size_t num_eligible_chunks;
} BMEM;

BMEM *bmem_new(size_t block_size);
void bmem_del(BMEM *mem);
void *bmem_block_new(BMEM *mem);
void bmem_block_del(BMEM *mem, void *block);

#ifdef LIBFLUX_BUILD
# include "bt.h"
#else
# include <flux/bt.h>
#endif

#endif /* BALLOC_H */
